@@ -31,7 +31,7 @@
 #ifdef HAVE_CAIRO_PS
 #include <cairo-ps.h>
 #endif
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 
 #include "ev-poppler.h"
 #include "ev-file-exporter.h"
@@ -228,6 +228,7 @@ static void
 convert_error (GError  *poppler_error,
 	       GError **error)
 {
+	g_return_if_fail (poppler_error != NULL);
 	if (poppler_error == NULL)
 		return;
 
@@ -239,12 +240,12 @@ convert_error (GError  *poppler_error,
 		else if (poppler_error->code == POPPLER_ERROR_ENCRYPTED)
 			code = EV_DOCUMENT_ERROR_ENCRYPTED;
 			
+		g_set_error_literal (error,
+                                     EV_DOCUMENT_ERROR,
+                                     code,
+                                     poppler_error->message);
 
-		g_set_error (error,
-			     EV_DOCUMENT_ERROR,
-			     code,
-			     poppler_error->message,
-			     NULL);
+		g_error_free (poppler_error);
 	} else {
 		g_propagate_error (error, poppler_error);
 	}
