@@ -26,7 +26,7 @@
 
 #include <gio/gio.h>
 #include <glib/gstdio.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
 
 #include "ev-backends-manager.h"
@@ -104,81 +104,45 @@ get_compression_from_mime_type (const gchar *mime_type)
 	return EV_COMPRESSION_NONE;
 }
 
-static gchar *
-get_mime_type_from_uri (const gchar *uri, GError **error)
-{
-	GFile     *file;
-	GFileInfo *file_info;
-	gchar     *mime_type;
-
-	file = g_file_new_for_uri (uri);
-	file_info = g_file_query_info (file,
-				       G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
-				       0, NULL, error);
-	g_object_unref (file);
-
-	if (file_info == NULL)
-		return NULL;
-
-	mime_type = g_strdup (g_file_info_get_content_type (file_info));
-	g_object_unref (file_info);
-
-	return mime_type;
-}
-
-static gchar *
-get_mime_type_from_data (const gchar *uri, GError **error)
-{
-	GFile            *file;
-	GFileInputStream *input_stream;
-	gssize            size_read;
-	guchar            buffer[1024];
-
-	file = g_file_new_for_uri (uri);
-	
-	input_stream = g_file_read (file, NULL, error);
-	if (!input_stream) {
-		g_object_unref (file);
-		return NULL;
-	}
-
-	size_read = g_input_stream_read (G_INPUT_STREAM (input_stream),
-					 buffer, 1024, NULL, NULL);
-	g_input_stream_close (G_INPUT_STREAM (input_stream), NULL, error);
-
-	g_object_unref (file);
-
-	if (size_read == -1)
-		return NULL;
-
-	return g_content_type_guess (NULL, /* no filename */
-				     buffer, 1024,
-				     NULL);
-}
 
+/*
+ * get_document_from_uri:
+ * @uri: the document URI
+ * @fast: whether to use fast MIME type detection
+ * @compression: a location to store the document's compression type
+ * @error: a #GError location to store an error, or %NULL
+ *
+ * Creates a #EvDocument instance for the document at @uri, using either
+ * fast or slow MIME type detection. If a document could be created,
+ * @compression is filled in with the document's compression type.
+ * On error, %NULL is returned and @error filled in.
+ * 
+ * Returns: a new #EvDocument instance, or %NULL on error with @error filled in
+ */
 static EvDocument *
 get_document_from_uri (const char        *uri,
-		       gboolean           slow,
+		       gboolean           fast,
 		       EvCompressionType *compression,
 		       GError           **error)
 {
 	EvDocument *document = NULL;
 	gchar      *mime_type = NULL;
+	GError     *err = NULL;
 
 	*compression = EV_COMPRESSION_NONE;
 
-	mime_type = slow ?
-		get_mime_type_from_data (uri, error) :
-		get_mime_type_from_uri (uri, error);
+	mime_type = ev_file_get_mime_type (uri, fast, &err);
 
 	if (mime_type == NULL) {
 		g_free (mime_type);
 
-		if (*error == NULL) {
-			g_set_error (error,
-				     EV_DOCUMENT_ERROR,
-				     0,
-				     _("Unknown MIME Type"));
+		if (err == NULL) {
+			g_set_error_literal (error,
+                                             EV_DOCUMENT_ERROR,
+                                             EV_DOCUMENT_ERROR_INVALID,
+                                             _("Unknown MIME Type"));
+		} else {
+			g_propagate_error (error, err);
 		}
 		
 		return NULL;
@@ -192,15 +156,19 @@ get_document_from_uri (const char        *uri,
 #endif /* ENABLE_PIXBUF */
 
 	if (document == NULL) {
-		gchar *mime_desc;
+		gchar *content_type, *mime_desc = NULL;
+
+		content_type = g_content_type_from_mime_type (mime_type);
+		if (content_type)
+			mime_desc = g_content_type_get_description (content_type);
 
-		mime_desc = g_content_type_get_description (mime_type);
 		g_set_error (error,
 			     EV_DOCUMENT_ERROR,	
-			     0,
+			     EV_DOCUMENT_ERROR_INVALID,
 			     _("File type %s (%s) is not supported"),
-			     mime_desc, mime_type);
+			     mime_desc ? mime_desc : "-", mime_type);
 		g_free (mime_desc);
+		g_free (content_type);
 		g_free (mime_type);
 
 		return NULL;
@@ -223,6 +191,17 @@ free_uncompressed_uri (gchar *uri_unc)
 	g_free (uri_unc);
 }
 
+/**
+ * ev_document_factory_get_document:
+ * @uri: an URI
+ * @error: a #GError location to store an error, or %NULL
+ *
+ * Creates a #EvDocument for the document at @uri; or, if no backend handling
+ * the document's type is found, or an error occurred on opening the document,
+ * returns %NULL and fills in @error.
+ *
+ * Returns: a new #EvDocument, or %NULL.
+ */
 EvDocument *
 ev_document_factory_get_document (const char *uri, GError **error)
 {
@@ -230,84 +209,88 @@ ev_document_factory_get_document (const char *uri, GError **error)
 	int result;
 	EvCompressionType compression;
 	gchar *uri_unc = NULL;
+	GError *err = NULL;
 
-	document = get_document_from_uri (uri, FALSE, &compression, error);
-	if (*error == NULL) {
-		uri_unc = ev_file_uncompress (uri, compression, error);
+	g_return_val_if_fail (uri != NULL, NULL);
+
+	document = get_document_from_uri (uri, TRUE, &compression, &err);
+	g_assert (document != NULL || err != NULL);
+
+	if (document != NULL) {
+		uri_unc = ev_file_uncompress (uri, compression, &err);
 		if (uri_unc) {
 			g_object_set_data_full (G_OBJECT (document),
 						"uri-uncompressed",
 						uri_unc,
 						(GDestroyNotify) free_uncompressed_uri);
-		}
-
-		if (*error != NULL) {
+		} else if (err != NULL) {
 			/* Error uncompressing file */
-			if (document)
-				g_object_unref (document);
+			g_object_unref (document);
+			g_propagate_error (error, err);
 			return NULL;
 		}
 
-		result = ev_document_load (document, uri_unc ? uri_unc : uri, error);
+		result = ev_document_load (document, uri_unc ? uri_unc : uri, &err);
 
-		if (result == FALSE || *error) {
-			if (*error &&
-			    (*error)->domain == EV_DOCUMENT_ERROR &&
-			    (*error)->code == EV_DOCUMENT_ERROR_ENCRYPTED)
+		if (result == FALSE || err) {
+			if (err &&
+			    g_error_matches (err, EV_DOCUMENT_ERROR, EV_DOCUMENT_ERROR_ENCRYPTED)) {
+				g_error_free (err);
 				return document;
+			    }
+			/* else fall through to slow mime code section below */
 		} else {
 			return document;
 		}
+
+		g_object_unref (document);
+		document = NULL;
 	}
 	
 	/* Try again with slow mime detection */
-	if (document)
-		g_object_unref (document);
-	document = NULL;
-
-	if (*error)
-		g_error_free (*error);
-	*error = NULL;
-
+	g_clear_error (&err);
 	uri_unc = NULL;
 
-	document = get_document_from_uri (uri, TRUE, &compression, error);
-
-	if (*error != NULL) {
+	document = get_document_from_uri (uri, FALSE, &compression, &err);
+	if (document == NULL) {
+		g_assert (err != NULL);
+		g_propagate_error (error, err);
 		return NULL;
 	}
 
-	uri_unc = ev_file_uncompress (uri, compression, error);
+	uri_unc = ev_file_uncompress (uri, compression, &err);
 	if (uri_unc) {
 		g_object_set_data_full (G_OBJECT (document),
 					"uri-uncompressed",
 					uri_unc,
 					(GDestroyNotify) free_uncompressed_uri);
-	}
-
-	if (*error != NULL) {
+	} else if (err != NULL) {
 		/* Error uncompressing file */
-		if (document)
-			g_object_unref (document);
+		g_propagate_error (error, err);
+
+		g_object_unref (document);
 		return NULL;
 	}
 	
-	result = ev_document_load (document, uri_unc ? uri_unc : uri, error);
-
+	result = ev_document_load (document, uri_unc ? uri_unc : uri, &err);
 	if (result == FALSE) {
-		if (*error == NULL) {
-			g_set_error (error,
-				     EV_DOCUMENT_ERROR,
-				     0,
-				     _("Unknown MIME Type"));
-		} else if ((*error)->domain == EV_DOCUMENT_ERROR &&
-			   (*error)->code == EV_DOCUMENT_ERROR_ENCRYPTED) {
+		if (err == NULL) {
+			/* FIXME: this really should not happen; the backend should
+			 * always return a meaningful error.
+			 */
+			g_set_error_literal (&err,
+                                             EV_DOCUMENT_ERROR,
+                                             EV_DOCUMENT_ERROR_INVALID,
+                                             _("Unknown MIME Type"));
+		} else if (g_error_matches (err, EV_DOCUMENT_ERROR, EV_DOCUMENT_ERROR_ENCRYPTED)) {
+			g_error_free (err);
 			return document;
 		}
 
-		if (document)
-			g_object_unref (document);
+		g_object_unref (document);
 		document = NULL;
+
+		g_propagate_error (error, err);
 	}
 	
 	return document;
@@ -343,7 +326,21 @@ file_filter_add_mime_types (EvTypeInfo *info, GtkFileFilter *filter)
 		gtk_file_filter_add_mime_type (filter, mime_type);
 }
 
-void 
+/**
+ * ev_document_factory_add_filters:
+ * @chooser: a #GtkFileChooser
+ * @document: a #EvDocument, or %NULL
+ *
+ * Adds some file filters to @chooser.
+ 
+ * Always add a "All documents" format.
+ * 
+ * If @document is not %NULL, adds a #GtkFileFilter for @document's MIME type.
+ *
+ * If @document is %NULL, adds a #GtkFileFilter for each document type that evince
+ * can handle.
+ */
+void
 ev_document_factory_add_filters (GtkWidget *chooser, EvDocument *document)
 {
 	GList         *all_types;
@@ -351,6 +348,9 @@ ev_document_factory_add_filters (GtkWidget *chooser, EvDocument *document)
 	GtkFileFilter *default_filter;
 	GtkFileFilter *document_filter;
 
+        g_return_if_fail (GTK_IS_FILE_CHOOSER (chooser));
+        g_return_if_fail (document == NULL || EV_IS_DOCUMENT (document));
+
 	all_types = ev_backends_manager_get_all_types_info ();
 	
 	default_filter = document_filter = filter = gtk_file_filter_new ();
